var ssKDReport = null;

function ssKDReportLoad() 
{
  ssKDReport = new ssKDReportManager();
  ssKDReport.init();
}

function ssKDReportUnload()
{
  ssKDReport.destroy();
  ssKDReport = null;
}

function ssKDReportManager()
{
  return this;
}

ssKDReportManager.prototype =
{
	mWords: "",
	mMatchWordCount: 1,
	
  init: function()
  {
  	var topWindow = getTopWindow();

  	var vbox = document.getElementById( "searchstatus-kdreport-details" );
  	var listbox = document.getElementById( "searchstatus-kdreport-listbox" );

		var selection = window.arguments[0];

		if( selection == null || selection.length == 0 )
		{
			selection = "No Keyword Selected";
		}
		
		selection = trim( selection );
		this.mMatchWordCount = this.countWords( selection );

		this.mWords = "";
		this.traverseNodes( topWindow._content.document.getElementsByTagName( "title" ) );

		var titlematchcount = this.countMatches( this.mWords, selection );		
		var titlewordcount = this.countWords( this.mWords );

		this.addRow( listbox, "Title", titlematchcount, titlewordcount );		

		this.mWords = "";
		this.traverse( topWindow._content.document.body );

		var totalmatchcount = this.countMatches( this.mWords, selection );		
		var totalwordcount = this.countWords( this.mWords );

		this.addRow( listbox, "Body", totalmatchcount, totalwordcount );
		
		this.mWords = "";
		this.traverseNodes( topWindow._content.document.getElementsByTagName( "h1" ) );
		this.traverseNodes( topWindow._content.document.getElementsByTagName( "h2" ) );
		this.traverseNodes( topWindow._content.document.getElementsByTagName( "h3" ) );

		var headingmatchcount = this.countMatches( this.mWords, selection );		
		var headingwordcount = this.countWords( this.mWords );

		this.addRow( listbox, "H1/H2/H3", headingmatchcount, headingwordcount );	
		
		this.mWords = "";
		this.traverseNodes( topWindow._content.document.getElementsByTagName( "a" ) );

		var linkmatchcount = this.countMatches( this.mWords, selection );		
		var linkwordcount = this.countWords( this.mWords );

		this.addRow( listbox, "Links", linkmatchcount, linkwordcount );		
		
		this.mWords = "";
		this.traverseNodes( topWindow._content.document.getElementsByTagName( "b" ) );
		this.traverseNodes( topWindow._content.document.getElementsByTagName( "strong" ) );

		var boldmatchcount = this.countMatches( this.mWords, selection );		
		var boldwordcount = this.countWords( this.mWords );

		this.addRow( listbox, "Bold", boldmatchcount, boldwordcount );				
		
		var metadescmatchcount = "n/a";
		var metadescwordcount = "n/a";
		
		var metakeywordsmatchcount = "n/a";
		var metakeywordswordcount = "n/a";
		
		var tags = topWindow._content.document.getElementsByTagName( "meta" );
		for( var i = 0; i < tags.length; i++ )
		{
			var tag = tags.item( i );
			if( tag.hasAttribute( "name" ) && tag.hasAttribute( "content" ) )
			{
				var name = tag.getAttribute( "name" ).toLowerCase();
				if( name == "description" )
				{
					this.mWords = tag.getAttribute( "content" );
					metadescmatchcount = this.countMatches( this.mWords, selection );		
					metadescwordcount = this.countWords( this.mWords );
				}
				else if( name == "keywords" )
				{
					this.mWords = tag.getAttribute( "content" );
					metakeywordsmatchcount = this.countMatches( this.mWords, selection );		
					metakeywordswordcount = this.countWords( this.mWords );
				}				
			}
		}

		this.addRow( listbox, "Meta description", metadescmatchcount, metadescwordcount );
		this.addRow( listbox, "Meta keywords", metakeywordsmatchcount, metakeywordswordcount );
		
		var dialog = document.getElementById( "searchstatus-kdreport-dialog" );
		if( selection.length > 20 )
		{
			selection = selection.substring( 0, 20 ) + "...";
		}
		
		dialog.setAttribute( "title", "Keyword Density Report for keyword '" + selection + "'" );
	},
	
  destroy: function()
  {
    
  },
  
  addRow: function( listbox, tag, matchCount, wordCount )
  {
		var listitem = document.createElement( "listitem" );
		
		var col1 = document.createElement( "listcell" );
		col1.setAttribute( "label", tag );
		col1.setAttribute( "class", "text" );
		listitem.appendChild( col1 );
		
		var col2 = document.createElement( "listcell" );
		col2.setAttribute( "label", matchCount );
		col2.setAttribute( "class", "text" );
		listitem.appendChild( col2 );	
		
		var col3 = document.createElement( "listcell" );
		col3.setAttribute( "label", wordCount );
		col3.setAttribute( "class", "text" );
		listitem.appendChild( col3 );			
		
		var percentage = ( wordCount == "n/a" ? "n/a" : ( wordCount == 0 ? 0 : Math.round( ( matchCount * this.mMatchWordCount * 1000 ) / wordCount ) / 10 ) );
		var col4 = document.createElement( "listcell" );
		col4.setAttribute( "label", percentage + ( wordCount == "n/a" ? "" : "%" ) );
		col4.setAttribute( "class", "text" );
		listitem.appendChild( col4 );				
		
		listbox.appendChild( listitem );
	},  	
	
	countMatches: function( text, str )
	{
		var wc = 0;
		
		var matchText = str.replace( /[ -]/g, "[ -]" );
		matchText = "\\b" + matchText + "\\b";
		
		var re = new RegExp( matchText, "gim" );
		var matches = text.match( re );
		if( matches != null ) 
		{
			wc = matches.length;
		}
		
		return wc;
	},	
	
	countWords: function( text )
	{
		var wc = 0;
		
		var matches = text.match( /\b\w+\b/g );
		if( matches != null ) 
		{
			wc = matches.length;
		}
		
		return wc;
	},

	traverseNodes: function( nodes )
	{
		if( nodes != null )
		{
			for( var i = 0; i < nodes.length; i++ ) 
			{
				this.traverse( nodes.item( i ) );
				this.mWords += '\n';
			}	
		}
	},
	
	traverse: function( node )
	{
		if( node.nodeType == 3 ) 
		{
			this.mWords += node.nodeValue;
		}
		
		var name = node.nodeName.toLowerCase();
		
		if( node.childNodes != null && name != 'script' && name != 'style' ) 
		{
			for( var i = 0; i < node.childNodes.length; i++ ) 
			{
				this.traverse( node.childNodes.item( i ) );
			}
		}
	},  
  
  copy: function()
  {
  	var listbox = document.getElementById( "searchstatus-kdreport-listbox" );
  	
  	var listitems = listbox.selectedItems;
  	var copytext = "";
  	
  	for( i = 0; i < listitems.length; i++ )
		{
			var item = listitems[i];

			copytext += item.childNodes.item( 0 ).getAttribute( "label" ) + ": " + item.childNodes.item( 1 ).getAttribute( "label" ) + ": " + item.childNodes.item( 2 ).getAttribute( "label" ) + ": " + item.childNodes.item( 3 ).getAttribute( "label" );
			copytext += "\n";
		}
		
		var str = Components.classes["@mozilla.org/supports-string;1"].createInstance( Components.interfaces.nsISupportsString );
		if( !str )
		{
			return false;
		}
		str.data=copytext;
		
		var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance( Components.interfaces.nsITransferable );
		if( !trans )
		{
			return false;
		}
		
		trans.addDataFlavor( "text/unicode" );
		trans.setTransferData( "text/unicode", str, copytext.length * 2 );
		
		var clipid=Components.interfaces.nsIClipboard;
		var clip = Components.classes["@mozilla.org/widget/clipboard;1"].getService( clipid );
		if( !clip )
		{
			return false;
		}
		
		clip.setData( trans, null, clipid.kGlobalClipboard );
		
		return true;
  }
};
  	
function getTopWindow()
{
  var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
  var windowManagerInterface = windowManager.QueryInterface( Components.interfaces.nsIWindowMediator );
  var topWindow = windowManagerInterface.getMostRecentWindow( "navigator:browser" );
  if( !topWindow )
  {
    topWindow = window.openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", "about:blank", null, null);
  }

  return topWindow;
} 

function trim(str) 
{
	return str.replace( /^\s*|\s*$/g, "" );
}

function debugsk( message ) 
{
  var time = new Date();
  var time_string = time.getHours() + ":" + time.getMinutes() + ":" + time.getSeconds();

  dump( "[SearchStatus " + time_string + "]: " + message + "\n" );
}
